@@ -708,7 +708,7 @@ bool FormWidgetSignature::updateOffsets(FILE *f, Goffset objStart, Goffset objEn
         return false;
     }
 
-    const size_t bufSize = static_cast<int>(objEnd - objStart);
+    const size_t bufSize = static_cast<size_t>(objEnd - objStart);
     if (Gfseek(f, objStart, SEEK_SET) != 0) {
         return false;
     }
@@ -2018,6 +2018,26 @@ void FormFieldSignature::setCustomAppearanceContent(const GooString &s)
     customAppearanceContent = GooString(s.toStr());
 }
 
+const GooString &FormFieldSignature::getCustomAppearanceLeftContent() const
+{
+    return customAppearanceLeftContent;
+}
+
+void FormFieldSignature::setCustomAppearanceLeftContent(const GooString &s)
+{
+    customAppearanceLeftContent = GooString(s.toStr());
+}
+
+double FormFieldSignature::getCustomAppearanceLeftFontSize() const
+{
+    return customAppearanceLeftFontSize;
+}
+
+void FormFieldSignature::setCustomAppearanceLeftFontSize(double size)
+{
+    customAppearanceLeftFontSize = size;
+}
+
 void FormFieldSignature::setCertificateInfo(std::unique_ptr<X509CertificateInfo> &certInfo)
 {
     certificate_info.swap(certInfo);
