@@ -34,7 +34,7 @@
 // Copyright (C) 2015 Li Junling <lijunling@sina.com>
 // Copyright (C) 2015 André Guerreiro <aguerreiro1985@gmail.com>
 // Copyright (C) 2015 André Esser <bepandre@hotmail.com>
-// Copyright (C) 2016 Jakub Alba <jakubalba@gmail.com>
+// Copyright (C) 2016, 2020 Jakub Alba <jakubalba@gmail.com>
 // Copyright (C) 2017 Jean Ghali <jghali@libertysurf.fr>
 // Copyright (C) 2017 Fredrik Fornwall <fredrik@fornwall.net>
 // Copyright (C) 2018 Ben Timby <btimby@gmail.com>
@@ -44,6 +44,10 @@
 // Copyright (C) 2018 Philipp Knechtges <philipp-dev@knechtges.com>
 // Copyright (C) 2019 Christian Persch <chpe@src.gnome.org>
 // Copyright (C) 2020 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright (C) 2020 Thorsten Behrens <Thorsten.Behrens@CIB.de>
+// Copyright (C) 2020 Adam Sampson <ats@offog.org>
+// Copyright (C) 2021 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2021 Mahmoud Khalil <mahmoudkhalil11@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -133,7 +137,7 @@ PDFDoc::PDFDoc()
     init();
 }
 
-PDFDoc::PDFDoc(const GooString *fileNameA, const GooString *ownerPassword, const GooString *userPassword, void *guiDataA)
+PDFDoc::PDFDoc(const GooString *fileNameA, const GooString *ownerPassword, const GooString *userPassword, void *guiDataA, const std::function<void()> &xrefReconstructedCallback)
 {
 #ifdef _WIN32
     int n, i;
@@ -173,11 +177,11 @@ PDFDoc::PDFDoc(const GooString *fileNameA, const GooString *ownerPassword, const
     // create stream
     str = new FileStream(file, 0, false, file->size(), Object(objNull));
 
-    ok = setup(ownerPassword, userPassword);
+    ok = setup(ownerPassword, userPassword, xrefReconstructedCallback);
 }
 
 #ifdef _WIN32
-PDFDoc::PDFDoc(wchar_t *fileNameA, int fileNameLen, GooString *ownerPassword, GooString *userPassword, void *guiDataA)
+PDFDoc::PDFDoc(wchar_t *fileNameA, int fileNameLen, GooString *ownerPassword, GooString *userPassword, void *guiDataA, const std::function<void()> &xrefReconstructedCallback)
 {
     OSVERSIONINFO version;
     int i;
@@ -214,11 +218,11 @@ PDFDoc::PDFDoc(wchar_t *fileNameA, int fileNameLen, GooString *ownerPassword, Go
     // create stream
     str = new FileStream(file, 0, false, file->size(), Object(objNull));
 
-    ok = setup(ownerPassword, userPassword);
+    ok = setup(ownerPassword, userPassword, xrefReconstructedCallback);
 }
 #endif
 
-PDFDoc::PDFDoc(BaseStream *strA, const GooString *ownerPassword, const GooString *userPassword, void *guiDataA)
+PDFDoc::PDFDoc(BaseStream *strA, const GooString *ownerPassword, const GooString *userPassword, void *guiDataA, const std::function<void()> &xrefReconstructedCallback)
 {
 #ifdef _WIN32
     int n, i;
@@ -243,21 +247,23 @@ PDFDoc::PDFDoc(BaseStream *strA, const GooString *ownerPassword, const GooString
 #endif
     }
     str = strA;
-    ok = setup(ownerPassword, userPassword);
+    ok = setup(ownerPassword, userPassword, xrefReconstructedCallback);
 }
 
-bool PDFDoc::setup(const GooString *ownerPassword, const GooString *userPassword)
+bool PDFDoc::setup(const GooString *ownerPassword, const GooString *userPassword, const std::function<void()> &xrefReconstructedCallback)
 {
     pdfdocLocker();
 
     if (str->getLength() <= 0) {
         error(errSyntaxError, -1, "Document stream is empty");
+        errCode = errDamaged;
         return false;
     }
 
     str->setPos(0, -1);
     if (str->getPos() < 0) {
         error(errSyntaxError, -1, "Document base stream is not seekable");
+        errCode = errFileIO;
         return false;
     }
 
@@ -273,12 +279,12 @@ bool PDFDoc::setup(const GooString *ownerPassword, const GooString *userPassword
     bool wasReconstructed = false;
 
     // read xref table
-    xref = new XRef(str, getStartXRef(), getMainXRefEntriesOffset(), &wasReconstructed);
+    xref = new XRef(str, getStartXRef(), getMainXRefEntriesOffset(), &wasReconstructed, false, xrefReconstructedCallback);
     if (!xref->isOk()) {
         if (wasReconstructed) {
             delete xref;
             startXRefPos = -1;
-            xref = new XRef(str, getStartXRef(true), getMainXRefEntriesOffset(true), &wasReconstructed);
+            xref = new XRef(str, getStartXRef(true), getMainXRefEntriesOffset(true), &wasReconstructed, false, xrefReconstructedCallback);
         }
         if (!xref->isOk()) {
             error(errSyntaxError, -1, "Couldn't read xref table");
@@ -300,7 +306,7 @@ bool PDFDoc::setup(const GooString *ownerPassword, const GooString *userPassword
             // try one more time to construct the Catalog, maybe the problem is damaged XRef
             delete catalog;
             delete xref;
-            xref = new XRef(str, 0, 0, nullptr, true);
+            xref = new XRef(str, 0, 0, nullptr, true, xrefReconstructedCallback);
             catalog = new Catalog(this);
         }
 
@@ -1364,6 +1370,15 @@ void PDFDoc::writeObject(Object *obj, OutStream *outStr, XRef *xRef, unsigned in
     case objString:
         writeString(obj->getString(), outStr, fileKey, encAlgorithm, keyLength, ref);
         break;
+    case objHexString: {
+        const GooString *s = obj->getHexString();
+        outStr->printf("<");
+        for (int i = 0; i < s->getLength(); i++) {
+            outStr->printf("%02x", s->getChar(i) & 0xff);
+        }
+        outStr->printf("> ");
+        break;
+    }
     case objName: {
         GooString name(obj->getName());
         GooString *nameToPrint = name.sanitizedName(false /* non ps mode */);
@@ -1924,13 +1939,14 @@ Outline *PDFDoc::getOutline()
     return outline;
 }
 
-PDFDoc *PDFDoc::ErrorPDFDoc(int errorCode, const GooString *fileNameA)
+std::unique_ptr<PDFDoc> PDFDoc::ErrorPDFDoc(int errorCode, const GooString *fileNameA)
 {
+    // We cannot call std::make_unique here because the PDFDoc constructor is private
     PDFDoc *doc = new PDFDoc();
     doc->errCode = errorCode;
     doc->fileName = fileNameA;
 
-    return doc;
+    return std::unique_ptr<PDFDoc>(doc);
 }
 
 long long PDFDoc::strToLongLong(const char *s)
