@@ -3,6 +3,8 @@
  * Copyright (C) 2012, Tobias Koenig <tokoe@kdab.com>
  * Copyright (C) 2012, 2013 Fabio D'Urso <fabiodurso@hotmail.it>
  * Copyright (C) 2012, 2014, 2018-2020, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2020, Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
+ * Copyright (C) 2021, Oliver Sander <oliver.sander@tu-dresden.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +24,8 @@
 #ifndef _POPPLER_ANNOTATION_PRIVATE_H_
 #define _POPPLER_ANNOTATION_PRIVATE_H_
 
+#include <memory>
+
 #include <QtCore/QPointF>
 #include <QtCore/QSharedDataPointer>
 
@@ -37,6 +41,8 @@ class PDFRectangle;
 namespace Poppler {
 class DocumentData;
 
+PDFRectangle boundaryToPdfRectangle(::Page *pdfPage, const QRectF &r, int flags);
+
 class AnnotationPrivate : public QSharedData
 {
 public:
@@ -87,14 +93,13 @@ public:
 
     /* The following helpers only work if pdfPage is set */
     void flushBaseAnnotationProperties();
-    void fillNormalizationMTX(double MTX[6], int pageRotation) const;
     void fillTransformationMTX(double MTX[6]) const;
     QRectF fromPdfRectangle(const PDFRectangle &r) const;
     PDFRectangle boundaryToPdfRectangle(const QRectF &r, int flags) const;
     AnnotPath *toAnnotPath(const QVector<QPointF> &l) const;
 
     /* Scan page for annotations, parentId=0 searches for root annotations, subtypes empty means all subtypes */
-    static QList<Annotation *> findAnnotations(::Page *pdfPage, DocumentData *doc, const QSet<Annotation::SubType> &subtypes, int parentId = -1);
+    static std::vector<std::unique_ptr<Annotation>> findAnnotations(::Page *pdfPage, DocumentData *doc, const QSet<Annotation::SubType> &subtypes, int parentId = -1);
 
     /* Add given annotation to given page */
     static void addAnnotationToPage(::Page *pdfPage, DocumentData *doc, const Annotation *ann);
@@ -104,7 +109,7 @@ public:
 
     Ref pdfObjectReference() const;
 
-    Link *additionalAction(Annotation::AdditionalActionType type) const;
+    std::unique_ptr<Link> additionalAction(Annotation::AdditionalActionType type) const;
 };
 
 }
