@@ -8,9 +8,11 @@
  * Copyright (C) 2018, Andre Heinecke <aheinecke@intevation.de>
  * Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
  * Copyright (C) 2018 Chinmoy Ranjan Pradhan <chinmoyrp65@protonmail.com>
- * Copyright (C) 2018, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
+ * Copyright (C) 2018, 2020, 2021 Oliver Sander <oliver.sander@tu-dresden.de>
  * Copyright (C) 2019 João Netto <joaonetto901@gmail.com>
  * Copyright (C) 2020 David García Garzón <voki@canvoki.net>
+ * Copyright (C) 2020 Thorsten Behrens <Thorsten.Behrens@CIB.de>
+ * Copyright (C) 2020 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,13 +31,19 @@
 
 #include "poppler-qt6.h"
 
+#include <config.h>
+
 #include <QtCore/QSizeF>
+#include <QUrl>
 
 #include <Form.h>
 #include <Object.h>
 #include <Link.h>
 #include <SignatureInfo.h>
 #include <CertificateInfo.h>
+#ifdef ENABLE_NSS3
+#    include <SignatureHandler.h>
+#endif
 
 #include "poppler-form.h"
 #include "poppler-page-private.h"
@@ -213,16 +221,16 @@ void FormField::setPrintable(bool value)
     m_formData->fm->getWidgetAnnotation()->setFlags(flags);
 }
 
-Link *FormField::activationAction() const
+std::unique_ptr<Link> FormField::activationAction() const
 {
-    Link *action = nullptr;
     if (::LinkAction *act = m_formData->fm->getActivationAction()) {
-        action = PageData::convertLinkActionToLink(act, m_formData->doc, QRectF());
+        return PageData::convertLinkActionToLink(act, m_formData->doc, QRectF());
     }
-    return action;
+
+    return {};
 }
 
-Link *FormField::additionalAction(AdditionalActionType type) const
+std::unique_ptr<Link> FormField::additionalAction(AdditionalActionType type) const
 {
     Annot::FormAdditionalActionsType actionType = Annot::actionFieldModified;
     switch (type) {
@@ -240,27 +248,27 @@ Link *FormField::additionalAction(AdditionalActionType type) const
         break;
     }
 
-    Link *action = nullptr;
     if (std::unique_ptr<::LinkAction> act = m_formData->fm->getAdditionalAction(actionType)) {
-        action = PageData::convertLinkActionToLink(act.get(), m_formData->doc, QRectF());
+        return PageData::convertLinkActionToLink(act.get(), m_formData->doc, QRectF());
     }
-    return action;
+
+    return {};
 }
 
-Link *FormField::additionalAction(Annotation::AdditionalActionType type) const
+std::unique_ptr<Link> FormField::additionalAction(Annotation::AdditionalActionType type) const
 {
     ::AnnotWidget *w = m_formData->fm->getWidgetAnnotation();
     if (!w) {
-        return nullptr;
+        return {};
     }
 
     const Annot::AdditionalActionsType actionType = toPopplerAdditionalActionType(type);
 
-    Link *action = nullptr;
     if (std::unique_ptr<::LinkAction> act = w->getAdditionalAction(actionType)) {
-        action = PageData::convertLinkActionToLink(act.get(), m_formData->doc, QRectF());
+        return PageData::convertLinkActionToLink(act.get(), m_formData->doc, QRectF());
     }
-    return action;
+
+    return {};
 }
 
 FormFieldButton::FormFieldButton(DocumentData *doc, ::Page *p, ::FormWidgetButton *w) : FormField(std::make_unique<FormFieldData>(doc, p, w)) { }
@@ -571,6 +579,7 @@ public:
 
     EntityInfo issuer_info;
     EntityInfo subject_info;
+    QString nick_name;
     QByteArray certificate_der;
     QByteArray serial_number;
     QByteArray public_key;
@@ -584,6 +593,11 @@ public:
     bool is_null;
 };
 
+CertificateInfo::CertificateInfo() : d_ptr(new CertificateInfoPrivate())
+{
+    d_ptr->is_null = true;
+}
+
 CertificateInfo::CertificateInfo(CertificateInfoPrivate *priv) : d_ptr(priv) { }
 
 CertificateInfo::CertificateInfo(const CertificateInfo &other) : d_ptr(other.d_ptr) { }
@@ -650,6 +664,12 @@ QString CertificateInfo::subjectInfo(EntityInfoKey key) const
     }
 }
 
+QString CertificateInfo::nickName() const
+{
+    Q_D(const CertificateInfo);
+    return d->nick_name;
+}
+
 QDateTime CertificateInfo::validityStart() const
 {
     Q_D(const CertificateInfo);
@@ -726,6 +746,21 @@ QByteArray CertificateInfo::certificateData() const
     return d->certificate_der;
 }
 
+bool CertificateInfo::checkPassword(const QString &password) const
+{
+#ifdef ENABLE_NSS3
+    Q_D(const CertificateInfo);
+    SignatureHandler sigHandler(d->nick_name.toUtf8().constData(), SEC_OID_SHA256);
+    unsigned char buffer[5];
+    memcpy(buffer, "test", 5);
+    sigHandler.updateHash(buffer, 5);
+    std::unique_ptr<GooString> tmpSignature = sigHandler.signDetached(password.toUtf8().constData());
+    return tmpSignature.get() != nullptr;
+#else
+    return false;
+#endif
+}
+
 class SignatureValidationInfoPrivate
 {
 public:
@@ -897,14 +932,8 @@ SignatureValidationInfo FormFieldSignature::validate(ValidateOptions opt) const
     return validate(opt, QDateTime());
 }
 
-SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &validationTime) const
+static CertificateInfoPrivate *createCertificateInfoPrivate(const X509CertificateInfo *ci)
 {
-    FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
-    const time_t validationTimeT = validationTime.isValid() ? validationTime.toSecsSinceEpoch() : -1;
-    SignatureInfo *si = fws->validateSignature(opt & ValidateVerifyCertificate, opt & ValidateForceRevalidation, validationTimeT);
-
-    // get certificate info
-    const X509CertificateInfo *ci = si->getCertificateInfo();
     CertificateInfoPrivate *certPriv = new CertificateInfoPrivate;
     certPriv->is_null = true;
     if (ci) {
@@ -926,6 +955,8 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
         certPriv->subject_info.email_address = subjectInfo.email.c_str();
         certPriv->subject_info.org_name = subjectInfo.organization.c_str();
 
+        certPriv->nick_name = ci->getNickName().c_str();
+
         X509CertificateInfo::Validity certValidity = ci->getValidity();
         certPriv->validity_start = QDateTime::fromSecsSinceEpoch(certValidity.notBefore, Qt::UTC);
         certPriv->validity_end = QDateTime::fromSecsSinceEpoch(certValidity.notAfter, Qt::UTC);
@@ -941,6 +972,19 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
         certPriv->is_null = false;
     }
 
+    return certPriv;
+}
+
+SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &validationTime) const
+{
+    FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
+    const time_t validationTimeT = validationTime.isValid() ? validationTime.toSecsSinceEpoch() : -1;
+    SignatureInfo *si = fws->validateSignature(opt & ValidateVerifyCertificate, opt & ValidateForceRevalidation, validationTimeT);
+
+    // get certificate info
+    const X509CertificateInfo *ci = si->getCertificateInfo();
+    CertificateInfoPrivate *certPriv = createCertificateInfoPrivate(ci);
+
     SignatureValidationInfoPrivate *priv = new SignatureValidationInfoPrivate(CertificateInfo(certPriv));
     switch (si->getSignatureValStatus()) {
     case SIGNATURE_VALID:
@@ -1012,4 +1056,65 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
     return SignatureValidationInfo(priv);
 }
 
+bool hasNSSSupport()
+{
+#ifdef ENABLE_NSS3
+    return true;
+#else
+    return false;
+#endif
+}
+
+QVector<CertificateInfo> getAvailableSigningCertificates()
+{
+    QVector<CertificateInfo> vReturnCerts;
+
+#ifdef ENABLE_NSS3
+    std::vector<std::unique_ptr<X509CertificateInfo>> vCerts = SignatureHandler::getAvailableSigningCertificates();
+
+    for (auto &cert : vCerts) {
+        CertificateInfoPrivate *certPriv = createCertificateInfoPrivate(cert.get());
+        vReturnCerts.append(CertificateInfo(certPriv));
+    }
+#endif
+
+    return vReturnCerts;
+}
+
+QString POPPLER_QT6_EXPORT getNSSDir()
+{
+#ifdef ENABLE_NSS3
+    return QString::fromLocal8Bit(SignatureHandler::getNSSDir().c_str());
+#else
+    return QString();
+#endif
+}
+
+void setNSSDir(const QString &path)
+{
+#ifdef ENABLE_NSS3
+    if (path.isEmpty())
+        return;
+
+    GooString *goo = QStringToGooString(path);
+    SignatureHandler::setNSSDir(*goo);
+    delete goo;
+#else
+    (void)path;
+#endif
+}
+
+namespace {
+std::function<QString(const QString &)> nssPasswordCall;
+}
+
+void setNSSPasswordCallback(const std::function<char *(const char *)> &f)
+{
+#ifdef ENABLE_NSS3
+    SignatureHandler::setNSSPasswordCallback(f);
+#else
+    qWarning() << "setNSSPasswordCallback called but this poppler is built without NSS support";
+    (void)f;
+#endif
+}
 }
