@@ -15,35 +15,26 @@ private slots:
 
 void TestPageLayout::checkNone()
 {
-    Poppler::Document *doc;
-    doc = Poppler::Document::load(TESTDATADIR "/unittestcases/UseNone.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/UseNone.pdf");
     QVERIFY(doc);
 
     QCOMPARE(doc->pageLayout(), Poppler::Document::NoLayout);
-
-    delete doc;
 }
 
 void TestPageLayout::checkSingle()
 {
-    Poppler::Document *doc;
-    doc = Poppler::Document::load(TESTDATADIR "/unittestcases/FullScreen.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/FullScreen.pdf");
     QVERIFY(doc);
 
     QCOMPARE(doc->pageLayout(), Poppler::Document::SinglePage);
-
-    delete doc;
 }
 
 void TestPageLayout::checkFacing()
 {
-    Poppler::Document *doc;
-    doc = Poppler::Document::load(TESTDATADIR "/unittestcases/doublepage.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/doublepage.pdf");
     QVERIFY(doc);
 
     QCOMPARE(doc->pageLayout(), Poppler::Document::TwoPageRight);
-
-    delete doc;
 }
 
 QTEST_GUILESS_MAIN(TestPageLayout)
