@@ -13,7 +13,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2007-2008, 2010, 2012, 2015-2019 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010, 2012, 2015-2020 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2010 Mike Slegeir <tehpola@yahoo.com>
 // Copyright (C) 2010, 2013 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
@@ -28,7 +28,7 @@
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright (C) 2018 Thibaut Brard <thibaut.brard@gmail.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
-// Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2019, 2021 Oliver Sander <oliver.sander@tu-dresden.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -144,7 +144,7 @@ class SplashOutputDevNoText : public SplashOutputDev
 public:
     SplashOutputDevNoText(SplashColorMode colorModeA, int bitmapRowPadA, bool reverseVideoA, SplashColorPtr paperColorA, bool bitmapTopDownA = true)
         : SplashOutputDev(colorModeA, bitmapRowPadA, reverseVideoA, paperColorA, bitmapTopDownA) { }
-    ~SplashOutputDevNoText() override { }
+    ~SplashOutputDevNoText() override;
 
     void drawChar(GfxState *state, double x, double y, double dx, double dy, double originX, double originY, CharCode code, int nBytes, const Unicode *u, int uLen) override { }
     bool beginType3Char(GfxState *state, double x, double y, double dx, double dy, CharCode code, const Unicode *u, int uLen) override { return false; }
@@ -153,11 +153,14 @@ public:
     void endTextObject(GfxState *state) override { }
     bool interpretType3Chars() override { return false; }
 };
+
+SplashOutputDevNoText::~SplashOutputDevNoText() = default;
+
 #endif
 
 int main(int argc, char *argv[])
 {
-    PDFDoc *doc = nullptr;
+    std::unique_ptr<PDFDoc> doc;
     GooString *fileName = nullptr;
     GooString *docTitle = nullptr;
     GooString *author = nullptr, *keywords = nullptr, *subject = nullptr, *date = nullptr;
@@ -359,7 +362,7 @@ int main(int argc, char *argv[])
         SplashImageFileFormat format = strcmp(extension, "jpg") ? splashFormatPng : splashFormatJpeg;
 
         splashOut = new SplashOutputDevNoText(splashModeRGB8, 4, false, color);
-        splashOut->startDoc(doc);
+        splashOut->startDoc(doc.get());
 
         for (int pg = firstPage; pg <= lastPage; ++pg) {
             InMemoryFile imf;
@@ -389,14 +392,13 @@ int main(int argc, char *argv[])
         delete htmlOut;
         delete htmlFileName;
         delete fileName;
-        delete doc;
         return -1;
 #endif
     }
 
     if (htmlOut->isOk()) {
         doc->displayPages(htmlOut, firstPage, lastPage, 72 * scale, 72 * scale, 0, true, false, false);
-        htmlOut->dumpDocOutline(doc);
+        htmlOut->dumpDocOutline(doc.get());
     }
 
     delete htmlOut;
@@ -405,8 +407,6 @@ int main(int argc, char *argv[])
 
     // clean up
 error:
-    if (doc)
-        delete doc;
     delete fileName;
 
     if (htmlFileName)
