@@ -25,7 +25,7 @@
 // Copyright (C) 2013 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
 // Copyright (C) 2014, 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
-// Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2019, 2021 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
 //
 // To see a description of the changes please see the Changelog file that
@@ -55,6 +55,7 @@
 #include "PSOutputDev.h"
 #include "Error.h"
 #include "Win32Console.h"
+#include "sanitychecks.h"
 
 #ifdef USE_CMS
 #    include <lcms2.h>
@@ -131,6 +132,14 @@ static GooString processcolorprofilename;
 static GfxLCMSProfilePtr processcolorprofile;
 #    endif
 #endif
+#ifdef USE_CMS
+static GooString defaultgrayprofilename;
+static GfxLCMSProfilePtr defaultgrayprofile;
+static GooString defaultrgbprofilename;
+static GfxLCMSProfilePtr defaultrgbprofile;
+static GooString defaultcmykprofilename;
+static GfxLCMSProfilePtr defaultcmykprofile;
+#endif
 
 static const ArgDesc argDesc[] = { { "-f", argInt, &firstPage, 0, "first page to print" },
                                    { "-l", argInt, &lastPage, 0, "last page to print" },
@@ -160,6 +169,11 @@ static const ArgDesc argDesc[] = { { "-f", argInt, &firstPage, 0, "first page to
 #    ifdef USE_CMS
                                    { "-processcolorprofile", argGooString, &processcolorprofilename, 0, "ICC color profile to use as the process color profile during rasterization and transparency reduction" },
 #    endif
+#endif
+#ifdef USE_CMS
+                                   { "-defaultgrayprofile", argGooString, &defaultgrayprofilename, 0, "ICC color profile to use as the DefaultGray color space" },
+                                   { "-defaultrgbprofile", argGooString, &defaultrgbprofilename, 0, "ICC color profile to use as the DefaultRGB color space" },
+                                   { "-defaultcmykprofile", argGooString, &defaultcmykprofilename, 0, "ICC color profile to use as the DefaultCMYK color space" },
 #endif
                                    { "-optimizecolorspace", argFlag, &optimizeColorSpace, 0, "convert gray RGB images to gray color space" },
                                    { "-passlevel1customcolor", argFlag, &passLevel1CustomColor, 0, "pass custom color in level1sep" },
@@ -185,7 +199,7 @@ static const ArgDesc argDesc[] = { { "-f", argInt, &firstPage, 0, "first page to
 
 int main(int argc, char *argv[])
 {
-    PDFDoc *doc;
+    std::unique_ptr<PDFDoc> doc;
     GooString *fileName;
     GooString *psFileName;
     PSLevel level;
@@ -197,7 +211,7 @@ int main(int argc, char *argv[])
     bool rasterAntialias = false;
     std::vector<int> pages;
 #ifdef USE_CMS
-    cmsColorSpaceSignature displayprofilecolorspace;
+    cmsColorSpaceSignature profilecolorspace;
 #endif
 
     Win32Console win32Console(&argc, &argv);
@@ -287,7 +301,7 @@ int main(int argc, char *argv[])
             processcolorformatspecified = true;
         } else {
             fprintf(stderr, "Error: Unknown process color format \"%s\".\n", processcolorformatname.c_str());
-            goto err05;
+            goto err1;
         }
     }
 
@@ -296,37 +310,37 @@ int main(int argc, char *argv[])
         processcolorprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(processcolorprofilename.c_str(), "r"));
         if (!processcolorprofile) {
             fprintf(stderr, "Error: Could not open the ICC profile \"%s\".\n", processcolorprofilename.c_str());
-            goto err05;
+            goto err1;
         }
         if (!cmsIsIntentSupported(processcolorprofile.get(), INTENT_RELATIVE_COLORIMETRIC, LCMS_USED_AS_OUTPUT) && !cmsIsIntentSupported(processcolorprofile.get(), INTENT_ABSOLUTE_COLORIMETRIC, LCMS_USED_AS_OUTPUT)
             && !cmsIsIntentSupported(processcolorprofile.get(), INTENT_SATURATION, LCMS_USED_AS_OUTPUT) && !cmsIsIntentSupported(processcolorprofile.get(), INTENT_PERCEPTUAL, LCMS_USED_AS_OUTPUT)) {
             fprintf(stderr, "Error: ICC profile \"%s\" is not an output profile.\n", processcolorprofilename.c_str());
-            goto err05;
+            goto err1;
         }
-        displayprofilecolorspace = cmsGetColorSpace(processcolorprofile.get());
-        if (displayprofilecolorspace == cmsSigCmykData) {
+        profilecolorspace = cmsGetColorSpace(processcolorprofile.get());
+        if (profilecolorspace == cmsSigCmykData) {
             if (!processcolorformatspecified) {
                 processcolorformat = splashModeCMYK8;
                 processcolorformatspecified = true;
             } else if (processcolorformat != splashModeCMYK8) {
                 fprintf(stderr, "Error: Supplied ICC profile \"%s\" is a CMYK profile, but process color format is not CMYK8.\n", processcolorprofilename.c_str());
-                goto err05;
+                goto err1;
             }
-        } else if (displayprofilecolorspace == cmsSigGrayData) {
+        } else if (profilecolorspace == cmsSigGrayData) {
             if (!processcolorformatspecified) {
                 processcolorformat = splashModeMono8;
                 processcolorformatspecified = true;
             } else if (processcolorformat != splashModeMono8) {
                 fprintf(stderr, "Error: Supplied ICC profile \"%s\" is a monochrome profile, but process color format is not monochrome.\n", processcolorprofilename.c_str());
-                goto err05;
+                goto err1;
             }
-        } else if (displayprofilecolorspace == cmsSigRgbData) {
+        } else if (profilecolorspace == cmsSigRgbData) {
             if (!processcolorformatspecified) {
                 processcolorformat = splashModeRGB8;
                 processcolorformatspecified = true;
             } else if (processcolorformat != splashModeRGB8) {
                 fprintf(stderr, "Error: Supplied ICC profile \"%s\" is a RGB profile, but process color format is not RGB.\n", processcolorprofilename.c_str());
-                goto err05;
+                goto err1;
             }
         }
     }
@@ -335,10 +349,31 @@ int main(int argc, char *argv[])
     if (processcolorformatspecified) {
         if (level1 && processcolorformat != splashModeMono8) {
             fprintf(stderr, "Error: Setting -level1 requires -processcolorformat MONO8");
-            goto err05;
+            goto err1;
         } else if ((level1Sep || level2Sep || level3Sep || overprint) && processcolorformat != splashModeCMYK8) {
             fprintf(stderr, "Error: Setting -level1sep/-level2sep/-level3sep/-overprint requires -processcolorformat CMYK8");
-            goto err05;
+            goto err1;
+        }
+    }
+#endif
+
+#ifdef USE_CMS
+    if (!defaultgrayprofilename.toStr().empty()) {
+        defaultgrayprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(defaultgrayprofilename.c_str(), "r"));
+        if (!checkICCProfile(defaultgrayprofile, defaultgrayprofilename.c_str(), LCMS_USED_AS_INPUT, cmsSigGrayData)) {
+            goto err1;
+        }
+    }
+    if (!defaultrgbprofilename.toStr().empty()) {
+        defaultrgbprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(defaultrgbprofilename.c_str(), "r"));
+        if (!checkICCProfile(defaultrgbprofile, defaultrgbprofilename.c_str(), LCMS_USED_AS_INPUT, cmsSigRgbData)) {
+            goto err1;
+        }
+    }
+    if (!defaultcmykprofilename.toStr().empty()) {
+        defaultcmykprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(defaultcmykprofilename.c_str(), "r"));
+        if (!checkICCProfile(defaultcmykprofile, defaultcmykprofilename.c_str(), LCMS_USED_AS_INPUT, cmsSigCmykData)) {
+            goto err1;
         }
     }
 #endif
@@ -420,7 +455,7 @@ int main(int argc, char *argv[])
     }
 
     // write PostScript file
-    psOut = new PSOutputDev(psFileName->c_str(), doc, nullptr, pages, mode, paperWidth, paperHeight, noCrop, duplex, /*imgLLXA*/ 0, /*imgLLYA*/ 0,
+    psOut = new PSOutputDev(psFileName->c_str(), doc.get(), nullptr, pages, mode, paperWidth, paperHeight, noCrop, duplex, /*imgLLXA*/ 0, /*imgLLYA*/ 0,
                             /*imgURXA*/ 0, /*imgURYA*/ 0, psRasterizeWhenNeeded, /*manualCtrlA*/ false, /*customCodeCbkA*/ nullptr, /*customCodeCbkDataA*/ nullptr, level);
     if (noCenter) {
         psOut->setPSCenter(false);
@@ -455,6 +490,11 @@ int main(int argc, char *argv[])
 #    ifdef USE_CMS
     psOut->setDisplayProfile(processcolorprofile);
 #    endif
+#endif
+#ifdef USE_CMS
+    psOut->setDefaultGrayProfile(defaultgrayprofile);
+    psOut->setDefaultRGBProfile(defaultrgbprofile);
+    psOut->setDefaultCMYKProfile(defaultcmykprofile);
 #endif
     psOut->setEmbedType1(!noEmbedT1Fonts);
     psOut->setEmbedTrueType(!noEmbedTTFonts);
@@ -487,8 +527,6 @@ int main(int argc, char *argv[])
 err2:
     delete psFileName;
 err1:
-    delete doc;
-err05:
     delete fileName;
 err0:
 
